# Copyright (c) 2017, Camptocamp SA
# All rights reserved.

# Auto-generated file. Do not Edit!
INFO = {
    "git_hash": "85647dd7755b7c763c87c4694afc0f4997a246a7",
    "version": "2.2",
    "git_tag": "2.2.0.dev15",
}
