.ui-widget-daredevel-checkboxTree li {
    list-style-type: none;
    position: relative;
}

.ui-widget-daredevel-checkboxTree li span {
    cursor: default;
    position: absolute;
    top: 1px;
    left: -16px;
}
